@@ -1,28 +1,93 @@
+# 
+# This file is part of CatalystX-ExtJS
+# 
+# This software is Copyright (c) 2010 by Moritz Onken.
+# 
+# This is free software, licensed under:
+# 
+#   The (three-clause) BSD License
+# 
 package CatalystX::Controller::ExtJS::REST;
-our $VERSION = '0.11';
-
-use base qw(Catalyst::Controller::REST);
-
-#with 'Catalyst::Component::InstancePerContext';
-
-use Config::Any;
-use Scalar::Util qw/ weaken /;
-use Carp qw/ croak /;
-
-use HTML::FormFu::ExtJS;
-use HTML::FormFu::Util qw( _merge_hashes );
-use Path::Class;
-use HTML::Entities;
-use JSON qw(encode_json);
-use Scalar::Util qw/ weaken /;
-
-use Lingua::EN::Inflect;
+BEGIN {
+  $CatalystX::Controller::ExtJS::REST::VERSION = '1.101560';
+}
+# ABSTRACT: RESTful interface to dbic objects
 
 use Moose;
+extends qw(Catalyst::Controller::REST);
+use MooseX::MethodAttributes;
 
+use Config::Any;
+use Scalar::Util ();
+use Carp qw/ croak /;
+use HTML::FormFu::ExtJS 0.076;
+use Path::Class;
+use HTML::Entities;
+use Lingua::EN::Inflect;
+use JSON::Any;
+
+use Moose::Util::TypeConstraints;
+subtype 'PathClassDir', as 'Path::Class::Dir';
+coerce 'PathClassDir', from 'ArrayRef', via { Path::Class::Dir->new( @{$_[0]} ) };
+coerce 'PathClassDir', from 'Str', via { Path::Class::Dir->new( $_[0] ) };
+subtype 'PathClassFile', as 'Path::Class::File';
+no Moose::Util::TypeConstraints;
+
+__PACKAGE__->config(
+    actions => {
+        begin => {
+            ActionClass => '+CatalystX::Action::ExtJS::Deserialize',
+        },
+        end => {
+            ActionClass => 'Serialize',
+        },
+        list => {
+            Chained        => '/', 
+            NSListPathPart => undef, 
+            Args           => undef, 
+            Direct         => undef, 
+            DirectArgs     => 1,
+        },
+        base => {
+            Chained     => '/',
+            NSPathPart  => undef,
+            CaptureArgs => 1,
+        },
+        object => {
+            Chained     => '/',
+            NSPathPart  => undef,
+            Args        => undef,
+            ActionClass => '+CatalystX::Action::ExtJS::REST',
+            Direct      => undef,
+        },
+        object_GET    => { Private => undef },
+        object_PUT    => { Private => undef },
+        object_POST   => { Private => undef },
+        object_DELETE => { Private => undef },
+    }
+);
 
 has '_extjs_config' => ( is => 'rw', isa => 'HashRef', builder => '_extjs_config_builder', lazy => 1 );
 
+has 'form_base_path' => ( is => 'rw', lazy_build => 1, isa => 'PathClassDir', coerce => 1 );
+
+has 'form_base_file' => ( is => 'rw', lazy_build => 1, isa => 'Path::Class::File' );
+
+has 'list_base_path' => ( is => 'rw', lazy_build => 1, isa => 'PathClassDir', coerce => 1 );
+
+has 'list_base_file' => ( is => 'rw', lazy_build => 1, isa => 'Path::Class::File' );
+
+has 'list_options_file' => ( is => 'rw', lazy_build => 1, isa => 'PathClassFile|Undef' );
+
+has 'form_config_cache' => ( is => 'rw', isa => 'HashRef', clearer => 'clear_form_config_cache', default => sub {{}});
+
+has 'default_resultset' => ( is => 'rw', isa => 'Str', lazy_build => 1 );
+
+# backwards compat
+sub base_file { shift->form_base_file(@_) };
+sub base_path { shift->form_base_path(@_) };
+
+
 sub _extjs_config_builder {
     my $self = shift;
     my $c = $self->_application;
@@ -34,8 +99,6 @@ sub _extjs_config_builder {
             schema => 'DBIC',
             resultset => $self->default_resultset
         },
-        form_base_path    => [qw(root forms)],
-        list_base_path    => [qw(root lists)],
         default_rs_method => 'extjs_rest_'.$default_rs_method,
         context_stash     => 'context',
         list_options_validation => {
@@ -46,6 +109,7 @@ sub _extjs_config_builder {
                 { name => 'sort' },
             ]
         },
+        find_method => 'find',
     };
     my $self_config   = $self->config || {};
     my $parent_config = $c->config->{'ControllerX::ExtJS::REST'} || {};
@@ -58,13 +122,37 @@ sub _extjs_config_builder {
     
 }
 
-sub is_extjs_upload {
-    my ( $self, $c ) = @_;
-    return ( $c->req->param('x-requested-by') && $c->req->param('x-requested-by') eq "ExtJS"
-          && $c->req->header('Content-Type') && $c->req->header('Content-Type') =~ /^multipart\/form-data/ );
+sub _build_form_base_path {
+    my $self = shift;
+    return Path::Class::Dir->new( $self->_app->path_to(qw(root forms)) );
 }
 
-sub default_resultset {
+sub _build_form_base_file {
+    my $self = shift;
+    my @path = split( /\//, $self->action_namespace );
+    return $self->form_base_path->file((pop @path) . '.yml');
+}
+
+sub _build_list_base_path {
+    my $self = shift;
+    return Path::Class::Dir->new( $self->_app->path_to(qw(root lists)) );
+}
+
+sub _build_list_base_file {
+    my $self = shift;
+    my @path = split( /\//, $self->action_namespace );
+    my $file = $self->list_base_path->file((pop @path) . '.yml');
+    return -e $file ? $file : $self->form_base_file;
+}
+
+sub _build_list_options_file {
+    my $self = shift;
+    my @path = split( /\//, $self->action_namespace );
+    my $file = $self->list_base_path->file((pop @path) . '_options.yml');
+	return -e $file ? $file : undef;
+}
+
+sub _build_default_resultset {
     my ($self, $c) = @_;
     my $class = ref $self;
     my $prefix;
@@ -76,11 +164,16 @@ sub default_resultset {
     return $prefix;
 }
 
+sub clear_caches {
+	my ($self) = @_;
+	$self->form_config_cache({});
+}
+
 sub validate_options {
     my ($self, $c) = @_;
     my $form = HTML::FormFu::ExtJS->new;
     $form->populate($self->_extjs_config->{list_options_validation});
-    if(-e $self->list_options_file) {
+    if($self->list_options_file) {
         $c->log->debug('found configuration file for parameters') if($c->debug);
         $form->load_config_file($self->list_options_file);
     }
@@ -88,11 +181,10 @@ sub validate_options {
     return $form;
 }
 
-sub list : Chained('/') NSListPathPart Args {
+sub list {
     my ( $self, $c ) = @_;
-
-    my $form = $self->get_form($c);
-    $form->load_config_file($self->list_base_file);
+	$self->clear_caches if($c->debug);
+    my $form = $self->get_form($c, $self->list_base_file);
     my $config = $form->model_config;
     croak "Need resultset and schema" unless($config->{resultset} && $config->{schema});
     my $model = join('::', $config->{schema}, $config->{resultset});
@@ -102,45 +194,59 @@ sub list : Chained('/') NSListPathPart Args {
     if($validate_options->has_errors) {
         $self->status_bad_request($c, message => 'One ore more parameters did not pass the validation');
         $c->stash->{rest} = { errors => $validate_options->validation_response->{errors} };
-        $c->detach;
         return;
     }
 
     my $rs = $c->model($model);
     $rs = $self->paging_rs($c, $form, $rs);
-    my @args = @{$c->req->args};
-    unshift(@args, $self->_extjs_config->{default_rs_method});
-    for my $rs_method (@args) {
-        next unless($rs_method);
-        if($rs_method && $rs_method ne "all" && DBIx::Class::ResultSet->can($rs_method)) {
-            $c->log->warn('Possibly malicious method "'.$rs_method.'" on resultset '.$rs_method.' has not been called');
+    
+    # collect rs methods from URI, body and request param
+    my @args = map {$_ => [] } @{$c->req->args};
+    my $data = $c->req->data;
+    if($data && (ref $data eq 'ARRAY') && ref $data->[0] eq 'ARRAY') {
+        $data = $data->[0];
+        for(my $i = 0; $i < @$data; $i++) {
+            my $argv = $data->[$i+1] if(ref $data->[$i+1]);
+            $argv = [$argv] unless(ref $argv eq 'ARRAY');
+            push(@args, $data->[$i], $argv);
+            $i++ if(ref $data->[$i+1]);
+        }
+    }
+    push(@args, map { $_ => [] } $c->req->param('resultset'));
+    unshift(@args, $self->_extjs_config->{default_rs_method} => []);
+    
+    for(my $i = 0; $i < @args; $i+=2) {
+        next unless(my $rs_method = $args[$i]);
+        my ($m, @params) = split(/,/, $rs_method);
+        if($rs_method && DBIx::Class::ResultSet->can($m)) {
+            $c->log->warn('Possibly malicious method "'.$m.'" on resultset '.$m.' has not been called');
             next;
         }
-        my ($m, @params) = split(/,/, $rs_method);
+        push(@params, @{$args[$i+1]});
         if($rs->can($m)) {
             if($c->debug) {
                 my $debug = qq(Calling resultset method $m);
                 $debug .= q( with arguments ').join(q(', '), @params).q(') if(@params);
                 $c->log->debug($debug);
             }
-            $rs = $rs->$m($c,@params);
+            $rs = $rs->$m($c, @params);
         } elsif($c->debug) {
             $c->log->debug(qq(Resultset method $m could not be found));
         }
     }
     
-    my $grid_data = $form->grid_data([$rs->all]);
+    my ($pk, $too_much) = $rs->result_source->primary_columns;
+    
+    my $grid_data = $form->grid_data([$rs->all], {metaData => {idProperty => $pk, messageProperty => 'message' }});
     if ($self->_extjs_config->{no_list_metadata}) {
         delete $grid_data->{metaData};
     }
     my $count = $rs->search(undef, { rows => undef, offset => undef })->count;
     $grid_data->{results} = $count;
-    
     $self->status_ok( $c, entity => $grid_data);
-    # list
 }
 
-sub paging_rs : Private {
+sub paging_rs {
     my ($self, $c, $form, $rs) = @_;
     my $params = $c->req->params;
     
@@ -158,30 +264,45 @@ sub paging_rs : Private {
     undef $sort unless($form->get_all_element({ nested_name => $sort }));
     
     my $paged = $rs->search(undef, { offset => $start, rows => $limit || undef});
-    $paged = $paged->search(undef, { order_by => [ { $direction => 'me.' . $sort } ] })
+    my $me = $rs->current_source_alias;
+    $paged = $paged->search(undef, { order_by => [ { $direction => join('.', $me, $sort) } ] })
       if $sort;
     return $paged;
 }
 
-sub base : Chained('/') NSPathPart CaptureArgs(1) {
+sub base {
     my ( $self, $c, $id ) = @_;
     $self->object($c, $id);
 }
 
-sub object : Chained('/') NSPathPart Args ActionClass('REST') {
-    my ( $self, $c, $id ) = @_;
 
-        
-    croak $self->base_file." cannot be found" unless(-e $self->base_file);
-    
-    my $config = Config::Any->load_files( {files => [ $self->base_file ], use_ext => 1, flatten_to_hash => 0 } );
-    $config = { %{$self->_extjs_config->{model_config}}, %{$config->{$self->base_file}->{model_config} || {}} };
+sub object {
+    my ( $self, $c, $id ) = @_;
+	croak $self->form_base_file." cannot be found" unless(-e $self->form_base_file);
+	$self->clear_caches if($c->debug);
+    my $config = $self->load_config_file($self->form_base_file);
+    $config = { %{$self->_extjs_config->{model_config}}, %{$config->{model_config} || {}} };
     $config->{resultset} ||= $self->default_resultset;
     croak "Need resultset and schema" unless($config->{resultset} && $config->{schema});
     $c->stash->{extjs_formfu_model_config} = $config;
-    
+
     my $object = $c->model(join('::', $config->{schema}, $config->{resultset}));
-        
+
+    my $req_method = lc($c->req->method);
+    
+    unless(defined $id || $req_method ne 'put') {
+        my ($pk, $too_much) = $object->result_source->primary_columns;
+        croak 'Not able to process result classes with multiple primary keys' if($too_much);
+        $id = $c->req->param($pk);
+    }
+    
+    if(!defined $id && $req_method eq 'get') {
+        $c->forward('list');
+        return;
+    }
+    
+    my $guard = $c->model($config->{schema})->txn_scope_guard;
+    
     if(my $rs = $self->_extjs_config->{default_rs_method}) {
         if($object->can($rs)) {
             $c->log->debug(qq(Calling default resultset method $rs)) if($c->debug);
@@ -191,17 +312,27 @@ sub object : Chained('/') NSPathPart Args ActionClass('REST') {
         }
     }
 
-    # Get row object
-    my $method = $config->{find_method} || 'find';
+    my $method = $config->{find_method} || $self->_extjs_config->{find_method};
     if (defined $id && defined $object) {
+		$object = $object->search( undef, { for => 'update' })
+            if($req_method ne 'get');
         $object = $object->$method($id);
         $c->stash->{object} = $object;
     }
-    
+	
     $c->stash->{form} =
-      $self->get_form($c)
-      ->load_config_file($self->path_to_forms(lc($c->req->method)));
-
+      $self->get_form($c, $self->path_to_forms($req_method));
+    
+    if($req_method eq 'get') {
+        $c->forward('object_GET');
+    } elsif($req_method eq 'post') {
+        $c->forward('object_POST');
+    } elsif($req_method eq 'put') {
+        $c->forward('object_PUT');
+    } elsif($req_method eq 'delete') {
+        $c->forward('object_DELETE');
+    }
+    $guard->commit;
 }
 
 
@@ -217,8 +348,14 @@ sub object_PUT {
     }
 
     $self->object_PUT_or_POST($c, $form, $object);
+	
+	$form->model("DBIC")->default_values($object);
+	my $model = $form->model("HashRef");
+	$model->flatten(1);
+	$model->options(0);
+	my $req = { %{$model->create}, %{$c->req->params} }; 
     
-    $form->process( $c->req );
+    $form->process( $req );
     
     if ( $form->submitted_and_valid ) {
         my $row = $form->model->update($object);
@@ -241,23 +378,26 @@ sub object_PUT_or_POST {
 sub object_POST {
     my ( $self, $c ) = @_;
     my $form = $c->stash->{form};
-
+    
     $self->object_PUT_or_POST($c, $form);
-
+	
     $form->process( $c->req );
 
     if ( $form->submitted_and_valid ) {
-        my $row = $form->model->create;
+		my $row = $form->model->create;
         $self->handle_uploads($c, $row, $form);
         
         $c->stash->{object} = $row;
-
-        # get values from model
-        $self->status_created(
+        # get values from model and set the primary key
+		my ($pk, $too_much) = $row->result_source->primary_columns;
+		my $data = $form->form_data( $row );
+		$data->{data}->{$pk} = $row->$pk;
+		$self->status_created(
             $c,
-            location => $c->uri_for( '', $row->id ),
-            entity   => $form->form_data( $row )
+            location => $c->uri_for( '', $row->$pk ),
+            entity => $data
         );
+    
     }
     else {
         # return form values and error messages
@@ -292,59 +432,47 @@ sub object_DELETE {
 
 sub path_to_forms {
     my $self = shift;
-    my $file = Path::Class::File->new($self->base_path . '_' . (shift) . '.yml');
-    return -e $file ? $file : $self->base_file;
-}
-
-sub base_path {
-    my $self = shift;
-    return Path::Class::Dir->new( @{$self->_extjs_config->{form_base_path}}, split( /\//, $self->action_namespace ) );
-}
-
-sub base_file {
-    my $self = shift;
-    my @path = split( /\//, $self->action_namespace );
-    return $self->base_path->parent->file((pop @path) . '.yml');
-}
-
-sub list_base_path {
-    my $self = shift;
-    return Path::Class::Dir->new( @{$self->_extjs_config->{list_base_path}}, split( /\//, $self->action_namespace ) );
-}
-
-sub list_base_file {
-    my $self = shift;
-    my @path = split( /\//, $self->action_namespace );
-    my $file = $self->list_base_path->parent->file((pop @path) . '.yml');
-    return -e $file ? $file : $self->base_file;
-}
-
-sub list_options_file {
-    my $self = shift;
-    my @path = split( /\//, $self->action_namespace );
-    return $self->list_base_path->parent->file((pop @path) . '_options.yml');
+    my $file = Path::Class::File->new($self->form_base_path . '_' . (shift) . '.yml');
+    return -e $file ? $file : $self->form_base_file;
 }
 
 sub get_form {
-    my ($self, $c) = @_;
-    #return $self->_form if($self->_form);
-    my $form = HTML::FormFu::ExtJS->new();
-    $form->query_type('Catalyst');
-    my $model_stash = $self->_extjs_config->{model_stash};
-    $model_stash->{schema} ||= "DBIC";
-    for my $model ( keys %$model_stash ) {
-            $form->stash->{$model} = $c->model( $model_stash->{$model} );
-    }
-    $form->model_config($self->_extjs_config->{model_config});
-
+    my ($self, $c, $file) = @_;
+	my $form = HTML::FormFu::ExtJS->new();
+	$form->query_type('Catalyst');
+	my $model_stash = $self->_extjs_config->{model_stash};
+	$model_stash->{schema} ||= "DBIC";
+	for my $model ( keys %$model_stash ) {
+			$form->stash->{$model} = $c->model( $model_stash->{$model} );
+	}
+	$form->model_config($self->_extjs_config->{model_config});
+	
+	$file ||= $self->form_base_file;
+	my $config = $self->load_config_file($file);
+	$form->populate($config);	
     # To allow your form validation packages, etc, access to the catalyst
     # context, a weakened reference of the context is copied into the form's
     # stash.
+	
     my $context_stash = $self->_extjs_config->{context_stash};
     $form->stash->{$context_stash} = $c;
-    weaken( $form->stash->{$context_stash} );
+    Scalar::Util::weaken( $form->stash->{$context_stash} );
+	return $form;
+}
 
-    return $form;
+sub load_config_file {
+	my ($self, $file) = @_;
+	my $config;
+    unless($config = $self->form_config_cache->{$file . ""}) {
+		$config = Config::Any->load_files( {
+                    files => [$file],
+                    use_ext         => 1,
+                    driver_args => { General => { -UTF8 => 1 }, },
+        } );
+		( undef, $config ) = %{ $config->[0] };
+		$self->form_config_cache->{$file.""} = $config;
+	}
+	return $config;
 }
 
 sub handle_uploads {
@@ -359,15 +487,10 @@ sub handle_uploads {
 }
 
 
-sub begin : ActionClass('+CatalystX::Action::ExtJS::Deserialize') {
+sub end {
     my ( $self, $c ) = @_;
     $self->next::method($c);
-}
-
-sub end : ActionClass('Serialize') {
-    my ( $self, $c ) = @_;
-    $self->next::method($c);
-    if ( $self->is_extjs_upload($c) ) {
+    if ( $c->req->is_ext_upload ) {
         my $stash_key = (
               $self->config->{'serialize'}
             ? $self->config->{'serialize'}->{'stash_key'}
@@ -375,7 +498,7 @@ sub end : ActionClass('Serialize') {
           )
           || 'rest';
         my $output;
-        eval { $output = JSON->new->encode( $c->stash->{$stash_key} ); };
+        eval { $output = JSON::Any->new->encode( $c->stash->{$stash_key} ); };
 
         $c->res->content_type('text/html');
         $c->res->output( encode_entities($output) );
@@ -404,7 +527,9 @@ sub _parse_NSListPathPart_attr {
 
 1;
 
-__END__
+
+
+=pod
 
 =head1 NAME
 
@@ -412,7 +537,7 @@ CatalystX::Controller::ExtJS::REST - RESTful interface to dbic objects
 
 =head1 VERSION
 
-version 0.11
+version 1.101560
 
 =head1 SYNOPSIS
 
@@ -433,12 +558,162 @@ version 0.11
 This controller will make CRUD operations with ExtJS dead simple. Using REST you can update, create, remove, read and list
 objects which are retrieved via L<DBIx::Class>. 
 
+=head1 USAGE
+
+=head2 Set-up Form Configuration
+
+To use this controller, you need to set up at least one configuration file
+per controller
+
+If you create a controller C<MyApp::Controller::User>:
+
+  package MyApp::Controller::User;
+  
+  use Moose;
+  extends 'CatalystX::Controller::ExtJS::REST';
+  
+  1;
+
+You also need to create a file C<root/forms/user.yml>. To a more fine
+grained control over object creation, deletion, update or listing, you 
+have to create some more files.
+
+  root/
+       forms/
+             user.yml
+             user_get.yml
+             user_post.yml
+             user_put.yml
+       lists/
+             user.yml
+
+Only C<root/forms/user.yml> is required. All other files are optional. If ExtJS issues
+a GET request, this controller will first try to find the file C<root/forms/user_get.yml>.
+If this file does not exist, it will fall back to the so called I<base file>
+C<root/forms/user.yml>.
+
+This controller tries to guess the correct model and resultset. The model defaults
+to C<DBIC> and the resultset is derived from the name of the controller.
+In this example the controller uses the resultset C<< $c->model('DBIC::User') >>.
+
+You can override these values in the form config files:
+
+  ---
+    model_config:
+      resultset: User
+      schema: DBIC
+    elements:
+      - name: username
+      - name: password
+      - name: name
+      - name: forename
+      
+  # root/forms/user_put.yml
+  # make username and password required an object is created
+  ---
+    load_config_file: root/forms/user.yml
+	constraints:
+	  - type: Required
+	    name: username
+	  - type: Required
+	    name: password
+
+Now you can fire up your Catalyst app and you should see two new chained actions:
+
+  Loaded Chained actions:
+  ...
+  | /users/...                          | /user/list
+  | /user/...                           | /user/object
+
+=head2 Accessing objects
+
+To access an object, simply request the controller's url with the desired method.
+A C<POST> request to C</user> will create a new user object. The response will include
+the id of the new object. You can get the object by requesting C</user/$id> via GET
+or remove it by using the C<DELETE> method. 
+
+To update an object, use C<PUT>. PUT is special since it also allows for partial 
+submits. This means, that the object is loaded into the form before the request parameters
+are applied to it. You only need to send changed columns to the server.
+
+=head2 Accessing a list of objects
+
+You can access L<http://localhost:3000/users> or L<http://localhost:3000/user> to get a list 
+of users, which can be used to populate an ExtJS store. 
+If you access this URL with your browser you'll get a HTML representation of all users. 
+If you access using a XMLHttpRequest using ExtJS the returned
+value will be a valid JSON string. Listing objects is very flexible and can easily be extended.
+There is also a built-in validation for query parameters. By default the following 
+parameters are checked for sane defaults:
+
+=over
+
+=item * dir (either C<asc>, C<ASC>, C<desc> or C<DESC>)
+
+=item * limit (integer, range between 0 and 100)
+
+=item * start (positive integer)
+
+=back
+
+You can extend the validation of parameters by providing an additional file. Place it in
+C<root/lists/> and add the suffix C<_options> (e. g. C<root/lists/user_options.yml>). 
+You can overwrite or extend the validation configuration there.
+
+Any more attributes you add to the url will result in a call to the corresponding resultset.
+
+  # http://localhost:3000/users/active/
+  
+  $c->model('DBIC::Users')->active($c)->all;
+
+As you can see, the Catalyst context object is passed as first parameter.
+You can even supply arguments to that method using a comma separated list:
+
+  # http://localhost:3000/users/active,arg1,arg2/
+  
+  $c->model('DBIC::Users')->active($c, 'arg1', 'arg2')->all;
+
+You can chain those method calls to any length. Though, you cannot access resultset method which are
+inherited from L<DBIx::Class::ResultSet>. This is a security restriction because
+an attacker could call C<http://localhost:3000/users/delete> which will lead to 
+C<< $c->model('DBIC::Users')->delete >>. This would remove all rows from C<DBIC::Users>!
+
+To define a default resultset method which gets called every time the controller hits the
+result table, set:
+
+  __PACKAGE__->config({default_rs_method => 'restrict'});
+
+This will lead to the following chain:
+
+  # http://localhost:3000/users/active,arg1,arg2/
+  
+  $c->model('DBIC::Users')->restrict($c)->active($c, 'arg1', 'arg2')->all;
+
+  # same for GET, POST and PUT
+  # http://localhost:3000/user/1234
+  
+  $c->model('DBIC::Users')->restrict($c)->find(1234);
+
+The C<default_rs_method> defaults to the value of L</default_rs_method>. If it is not set 
+by the configuration, this controller tries to call C<extjs_rest_$class> (i.e. C<extjs_rest_user>).
+
+=head2 Handling Uploads
+
+This module handles your uploads. If there is an upload and the name of that field
+exists in you form config, the column is set to an L<IO::File> object. You need to
+handle this on the model side because storing a filehandle will most likely fail.
+
+Fortunately, there are modules out there which can help you with that. Have a look at
+L<DBIx::Class::InflateColumn::FS>. Don't use L<DBIx::Class:InflateColumn::File> 
+because it is deprecated and broken. 
+If you need a more advanced processing of uploaded files, don't hesitate and
+overwrite L</handle_uploads>.
+
 =head1 CONFIGURATION
 
 Local configuration:
-  
-  __PACKAGE__->config({ ... });  
 
+  __PACKAGE__->config({ ... });  
 
 Global configuration for all controllers which use CatalystX::Controller::ExtJS::REST:
 
@@ -453,7 +728,7 @@ The method to call on the resultset to get an existing row object.
 This can be set to the name of a custom function function which is defined with the (custom) resultset class.
 It needs to take the primary key as first parameter.
 
-Defaults to 'find'.
+Defaults to C<find>.
 
 =head2 default_rs_method
 
@@ -511,179 +786,60 @@ Defaults to L<Catalyst::Controller/namespace>
 
 Defaults to the plural form of L</namespace>. If this is the same as L</namespace> C<list_> is prepended.
 
-
 =head1 LIMITATIONS
 
 This module is limited to L<HTML::FormFu> as form processing engine,
 L<DBIx::Class> as ORM and L<Catalyst> as web application framework.
 
+=head1 PUBLIC ATTRIBUTES
 
+To change the default value of an attribute, either set it as default value
 
-=head1 USAGE
-
-=head2 Required Files
-
-Considering you create controller like this:
-
-  package MyApp::Controller::User;
+  package MyApp::Controller::MyController;
+  use Moose;
+  extends 'CatalystX::Controller::ExtJS::REST';
   
-  use base 'CatalystX::Controller::ExtJS::REST';
-  
-  1;
-  
-Then you will want to create the following files:
+  has '+default_result' => ( default => 'MyUser' );
 
-  root/
-       forms/
-             user.yml
-             user_get.yml
-             user_post.yml
-             user_put.yml
-       lists/
-             user.yml
+use the config
 
-Only C<root/forms/user.yml> is required. All other files are optional. If ExtJS issues
-a GET request, this controller will first try to find the file C<root/forms/user_get.yml>.
-If this file does not exist, it will fall back to the so called base file 
-C<root/forms/user.yml>.
+  __PACKAGE__->config( default_result => 'MyUser' );
 
-This controller tries to guess the correct model and resultset. The model defaults
-to C<DBIC> and the resultset is derived from the name of the controller.
-In this example we look for the resultset C<< $c->model('DBIC::User') >>.
+or overwrite the builder
 
-You can override these values in the form config files:
+  sub _build_default_result { return 'MyUser' };
 
-  # root/forms/user.yml
-  ---
-    elements:
-      - name: username
-      - name: password
-      - name: name
-      - name: forename
+=head2 default_resultset
 
-  # root/forms/user.yml (exactly the same as the above)
-  ---
-    model_config:
-      resultset: User
-      schema: DBIC
-    elements:
-      - name: username
-      - name: password
-      - name: name
-      - name: forename
-      
-  # root/forms/user_get.yml and friends
-  ---
-    load_config_file: root/forms/user.yml
+Determines the default name of the resultset class from the Model / View or
+Controller class if the forms contains no <model_config/resultset> config
+value. Defaults to the class name of the controller.
 
-Now you can fire up your Catalyst app and you should see two new chained actions:
+=head2 list_base_path
 
-  Loaded Chained actions:
-  ...
-  | /users/...                          | /user/list
-  | /user/...                           | /user/object
-    
-You can access L<http://localhost:3000/users> to get a list of users, which can be used
-to feed an ExtJS grid. If you access this URL with your browser you'll get a HTML 
-representation of all users. If you access using a XMLHttpRequest using ExtJS the returned
-value will be a valid JSON string. Listing objects is very flexible and can easily extended.
-There is also a built in validation for query parameters. By default the following 
-parameters are checked for sane defaults:
+Returns the path in which form config files for grids will be searched.
 
-=over
+=head2 list_base_file
 
-=item dir (either C<asc>, C<ASC>, c<desc> or C<DESC>)
+Returns the path to the specific form config file for grids or the default
+form config file if the specfic one can not be found.
 
-=item limit (integer, range between 0 and 100)
+=head2 path_to_forms
 
-=item start (positive integer)
+Returns the path to the specific form config file or the default form config
+file if the specfic one can not be found.
 
-=back
+=head2 form_base_path
 
-You can extend the validation of parameters by providing an additional file. Place it in
-C<root/lists/> and postfix it with C<_options> (e. g. C<root/lists/user_options.yml>). 
-You can overwrite or extend the validation configuration there.
+=head2 base_path
 
+Returns the path in which form config files will be searched.
 
-Any more attributes you add to the url will result in a call to the corresponding resultset.
+=head2 form_base_file
 
-  # http://localhost:3000/users/active/
-  
-  $c->model('DBIC::Users')->active($c)->all;
-  
-As you can see, the Catalyst context object is passed as first parameter.
-You can even supply arguments to that method using a komma separated list:
+=head2 base_file
 
-  # http://localhost:3000/users/active,arg1,arg2/
-  
-  $c->model('DBIC::Users')->active($c, 'arg1', 'arg2')->all;
-
-You can chain those method calls to any length. You cannot access resultset method which are
-inherited from L<DBIx::Class::ResultSet>, except C<all>. This is a security restriction because
-an attacker could call C<http://localhost:3000/users/delete> which will lead to 
-C<< $c->model('DBIC::Users')->delete >>. This would remove all rows from C<DBIC::Users>!
-
-To define a default resultset method which gets called every time the controller hits the
-result table, set:
-
-  __PACKAGE__->config({default_rs_method => 'restrict'});
-
-This will lead to the following chain:
-
-  # http://localhost:3000/users/active,arg1,arg2/
-  
-  $c->model('DBIC::Users')->restrict($c)->active($c, 'arg1', 'arg2')->all;
-
-  # and even with GET, POST and PUT
-  # http://localhost:3000/user/1234
-  
-  $c->model('DBIC::Users')->restrict($c)->find(1234);
-  
-The C<default_rs_method> defaults to the value of L</default_rs_method>. If it is not set 
-by the configuration, this controller tries to call C<extjs_rest_$class> (i.e. C<extjs_rest_user>).
-
-To create, delete and modify C<user> objects, simply C<POST>, C<DELETE> or C<PUT> to
-the url C</user>. C<POST> and C<DELETE> require that you add the id to that url,
-e. g. C</user/1234>.
-
-=head2 Handling Uploads
-
-This module handles your uploads. If there is an upload and the name of that field
-exists in you form config, the column is set to an L<IO::File> object. You need to
-handle this on the model side because storing a filehandle will most likely fail.
-
-There a modules out there which can help you with that. Have a look at
-L<DBIx::Class::InflateColumn::FS>. L<DBIx::Class:InflateColumn::File> will not work as this
-module expects a hash with the file handler and the file name set. But you can
-still overwrite L</handle_uploads> to your needs.
-
-As an upload field is a regular field it gets set twice. First the filename is set
-and C<< $row->update >> is called. This is entirely handled by L<HTML::FormFu::Model::DBIC>.
-After that L</handle_uploads> is called which sets the value of a upload field
-to the corresponding L<IO::File> object. Make sure you test for that, if you plan to
-inflate such a column.
-
-If you want to handle uploads yourself, overwrite L</handle_uploads>
-  
-  sub handle_uploads {
-      my ($self, $c, $row) = @_;
-      if(my $file = c->req->uploads->{upload}) {
-          $file->copy_to('yourdestination/'.$filename);
-          $row->upload($file->filename);
-      }
-  }
-
-But this should to be part of the model actually.
-
-Since you cannot upload files with an C<XMLHttpRequest> ExtJS creates an iframe and issues
-a C<POST> request in there. If you need to make a C<PUT> request you have to tunnel the
-desired method using a hidden field, by using the C<params> config option of 
-C<Ext.form.Action.Submit> or C<extraParams> in C<Ext.Ajax.request>. The name of that
-parameter has to be C<x-tunneled-method>.
-
-Make sure you do not include a file field in your C<GET> form definition. It will
-cause a security error in your browser because it is not allowed set the value of
-a file field.
+Returns the path to the default form config file.
 
 =head1 PUBLIC METHODS
 
@@ -701,26 +857,33 @@ List Action which returns the data for a ExtJS grid.
 Handles uploaded files by assigning the filehandle to the column accessor of
 the DBIC row object.
 
-=head2 is_extjs_upload
+As an upload field is a regular field it gets set twice. First the filename is set
+and C<< $row->update >> is called. This is entirely handled by L<HTML::FormFu::Model::DBIC>.
+After that L</handle_uploads> is called which sets the value of a upload field
+to the corresponding L<IO::File> object. Make sure you test for that, if you plan to
+inflate such a column.
 
-Returns true if the current request looks like a request from ExtJS and has
-multipart form data, so usually an upload. This requires that you add a C<x-requested-by> parameter to your
-form which has the value C<ExtJS>. This can be done either by adding a hidden form field,
-by using the C<params> config option of ExtJS C<Ext.form.Action.Submit> or C<extraParams> in C<Ext.Ajax.request>.
+If you want to handle uploads yourself, overwrite L</handle_uploads>
 
-=head2 default_resultset
+  sub handle_uploads {
+      my ($self, $c, $row) = @_;
+      if(my $file = c->req->uploads->{upload}) {
+          $file->copy_to('yourdestination/'.$filename);
+          $row->upload($file->filename);
+      }
+  }
 
-Determines the default name of the resultset class from the Model / View or
-Controller class.
+However, this should to be part of the model.
 
-=head2 list_base_path
+Since you cannot upload files with an C<XMLHttpRequest>, ExtJS creates an iframe and issues
+a C<POST> request in there. If you need to make a C<PUT> request you have to tunnel the
+desired method using a hidden field, by using the C<params> config option of 
+C<Ext.form.Action.Submit> or C<extraParams> in C<Ext.Ajax.request>. The name of that
+parameter has to be C<x-tunneled-method>.
 
-Returns the path in which form config files for grids will be searched.
-
-=head2 list_base_file
-
-Returns the path to the specific form config file for grids or the default
-form config file if the specfic one can not be found.
+Make sure you do not include a file field in your C<GET> form definition. It will
+cause a security error in your browser because it is not allowed set the value of
+a file field.
 
 =head2 object
 
@@ -750,19 +913,12 @@ REST Action to get the data of a single model entity with a GET request.
 
 REST Action to delete a single model entity with a DELETE request.
 
-=head2 path_to_forms
+=head1 PRIVATE ATTRIBUTES
 
-Returns the path to the specific form config file or the default form config
-file if the specfic one can not be found.
-
-=head2 base_path
-
-Returns the path in which form config files will be searched.
-
-=head2 base_file
-
-Returns the path to the default form config file.
+=head2 _extjs_config
 
+This attribute holds the configuration for the controller.
+It is created by merging by C<< __PACKAGE__->config >> with the default values.
 
 =head1 PRIVATE METHODS
 
@@ -773,8 +929,6 @@ These methods are private. Please don't overwrite those unless you know what you
 Run this code before any action in this controller. It sets the C<ActionClass> to L<CatalystX::Action::ExtJS::Deserialize>.
 This C<ActionClass> makes sure that no deserialization happens if the body's content is a file upload.
 
-=cut
-
 =head2 end
 
 If the request contains a file upload field, extjs expects the json response to be serialized and 
@@ -784,23 +938,25 @@ returned in a document with the C<Content-type> set to C<text/html>.
 
 =head2 _parse_NSListPathPart_attr
 
-=head2 _extjs_config
+=head1 CONTRIBUTORS
 
-This accessor contains the configuration options for this controller. It is created by merging
-C<< __PACKAGE__->config >> with the default values.
+  Mario Minati
 
 =head1 AUTHOR
 
-  Moritz Onken
-  
-  Mario Minati
+  Moritz Onken <onken@netcubed.de>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2009 by Moritz Onken.
+This software is Copyright (c) 2010 by Moritz Onken.
 
 This is free software, licensed under:
 
   The (three-clause) BSD License
 
-=cut
\ No newline at end of file
+=cut
+
+
+__END__
+
+
